//====================================================================
// NAME: cd_booth.v
// DESC: cordic booth radix-4 recoding unit
// HISTORY
// 11/26/2005: 
//	- first draft
//====================================================================




// data_in[31:30] can't be used.
// data_in[21:20] should be {0,0} for correct operation.

module Cd_booth_recoding(data_in,	//data_in[31:0]
			 i_in, 		//0,2,4,6,8,...,28  
			 i_next, 	//next i
			 i_xyz,		//i for shifter
			 sign,		//0:positive  1:negative
			 pass		//1:no_action 
);
//bit 31 30 29 28 27 26 ...
//i=   x  x  0  1  2  3 ...

input [31:0] data_in;
input [4:0] i_in;
output [4:0] i_next;
output [4:0] i_xyz;
output sign;
output pass;

wire [1:0] value_2928;
wire sign_2928;


wire [9:0] booth_nzero;
reg [4:0] zero_count;
wire [9:0] booth_tmp;

wire [31:0] booth_in;

assign booth_in = (data_in << i_in)|(32'h55555555>>(6'd32-{1'b0,i_in}));
// bit[29:29], 2 bits of booth_in will be used for i_yz and sign output.

Cd_booth_3bit booth_31(.data_in(booth_in[29:27]),
		       .value_out(value_2928),
		       .sign_out(sign_2928)
);

assign i_xyz = (value_2928==2'b01)?(i_in+5'd1):i_in;
assign pass = (value_2928==2'b00);
assign sign = sign_2928;
assign i_next=i_in+zero_count;


assign booth_nzero = booth_tmp;

Cd_booth_1bit booth_29(.data_in(booth_in[27:25]),
		       .nzero(booth_tmp[9])
);
Cd_booth_1bit booth_27(.data_in(booth_in[25:23]),
		       .nzero(booth_tmp[8])
);
Cd_booth_1bit booth_25(.data_in(booth_in[23:21]),
		       .nzero(booth_tmp[7])
);
Cd_booth_1bit booth_23(.data_in(booth_in[21:19]),
		       .nzero(booth_tmp[6])
);
Cd_booth_1bit booth_21(.data_in(booth_in[19:17]),
		       .nzero(booth_tmp[5])
);
Cd_booth_1bit booth_19(.data_in(booth_in[17:15]),
		       .nzero(booth_tmp[4])
);
Cd_booth_1bit booth_17(.data_in(booth_in[15:13]),
		       .nzero(booth_tmp[3])
);
Cd_booth_1bit booth_15(.data_in(booth_in[13:11]),
		       .nzero(booth_tmp[2])
);
Cd_booth_1bit booth_13(.data_in(booth_in[11:9]),
		       .nzero(booth_tmp[1])
);
Cd_booth_1bit booth_11(.data_in(booth_in[9:7]),
		       .nzero(booth_tmp[0])
);


always @(booth_nzero)
begin
    casex(booth_nzero)
    10'b1xxxxxxxxx: zero_count<=5'd2;
    10'b01xxxxxxxx: zero_count<=5'd4;
    10'b001xxxxxxx: zero_count<=5'd6;
    10'b0001xxxxxx: zero_count<=5'd8;
    10'b00001xxxxx: zero_count<=5'd10;
    10'b000001xxxx: zero_count<=5'd12;
    10'b0000001xxx: zero_count<=5'd14;
    10'b00000001xx: zero_count<=5'd16;
    10'b000000001x: zero_count<=5'd18;
    10'b0000000001: zero_count<=5'd20;
    10'b0000000000: zero_count<=5'd22;
    //default:        zero_count<=5'd22;
    endcase
end

endmodule




module Cd_booth_3bit(data_in,   //[2:0] [2:1]:current bit
		     value_out, //[1:0] 2,1,0
		     sign_out   //[2:0] 0:positive 1:negative
);
input [2:0] data_in;
output [1:0] value_out;
output sign_out;

reg [1:0] value_out;
reg sign_out;

always @(data_in)
begin
    case(data_in)
    3'b000:begin
	value_out=2'b00;
        sign_out=1'b0;
      end
    3'b001:begin
	value_out=2'b01;
        sign_out=1'b0;
      end
    3'b010:begin
	value_out=2'b01;
        sign_out=1'b0;
      end
    3'b011:begin
	value_out=2'b10;
        sign_out=1'b0;
      end
    3'b100:begin
	value_out=2'b10;
        sign_out=1'b1;
      end
    3'b101:begin
	value_out=2'b01;
        sign_out=1'b1;
      end
    3'b110:begin
	value_out=2'b01;
        sign_out=1'b1;
      end
    3'b111:begin
	value_out=2'b00;
        sign_out=1'b0;
      end
    endcase
end

endmodule //Cd_booth_3bit




module Cd_booth_1bit(data_in,   //[2:0] [2:1]:current bit
		     nzero
);
input [2:0] data_in;
output nzero;

reg nzero;

always @(data_in)
begin
    case(data_in)
    3'b000,3'b111:
	nzero=1'b0;
    default:
	nzero=1'b1;
    endcase
end

endmodule //Cd_booth_1bit




	




